"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
/**
 * Full formatter for log messages.
 * @param {object} cfg  Configuration object for the formatter.
 * @returns {FormatPlugin} The full formatter function.
 */
function fullFormatter(cfg = {}) {
    var _a;
    const fmCfg = {
        includeTimestamp: true,
        formatTimestamp: (timestamp) => timestamp.toISOString(),
        includeTags: true,
        includeMeta: true,
        separator: '\t',
        ...cfg
    };
    fmCfg.inspectOptions = {
        depth: Infinity,
        colors: true,
        ...((_a = fmCfg.inspectOptions) !== null && _a !== void 0 ? _a : {})
    };
    const fullFmt = (ctx) => {
        const msg = [];
        if (fmCfg.includeTimestamp) {
            msg.push(fmCfg.formatTimestamp(new Date()));
        }
        msg.push(ctx.level.toUpperCase(), ctx.msg);
        const parts = [
            msg.join(fmCfg.separator)
        ];
        if (fmCfg.includeTags && ctx.tags.length) {
            const tags = ctx.tags.map(tag => `${tag}`).join(', ');
            parts.push(`→ ${tags}`);
        }
        if (fmCfg.includeMeta && Object.keys(ctx.meta).length) {
            const meta = (0, util_1.inspect)(ctx.meta, fmCfg.inspectOptions);
            parts.push(`→ ${meta.replace(/\n/g, '\n  ')}`);
        }
        return parts.join('\n');
    };
    fullFmt._cfg = fmCfg;
    return fullFmt;
}
exports.default = fullFormatter;
