"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toBool = exports.stubError = exports.isError = void 0;
/**
 * Checks if value is an Error or Error-like object
 * @param {any} val The value to check
 * @returns {boolean} Whether the value is an Error or Error-like object
 */
function isError(val) {
    return Boolean(val) && typeof val === 'object' && (val instanceof Error || (Object.prototype.hasOwnProperty.call(val, 'message') &&
        Object.prototype.hasOwnProperty.call(val, 'stack')));
}
exports.isError = isError;
/**
 * Stubs an Error or Error-like object to include a toJSON method.
 * @param {Error} error The error to stub.
 * @returns {StubbedError} The original error object with a toJSON method.
 */
function stubError(error) {
    const err = error;
    if (typeof err.toJSON === 'function')
        return err;
    err.toJSON = function () {
        const keys = [
            'name',
            'message',
            'stack'
        ].concat(Object.keys(err));
        return keys.reduce((obj, key) => {
            /* istanbul ignore next */
            if (key in err) {
                const val = err[key];
                if (typeof val !== 'function') {
                    obj[key] = val;
                }
            }
            return obj;
        }, {});
    };
    return err;
}
exports.stubError = stubError;
/**
 * Converts a string or number value to a boolean.
 * @param {string|number|boolean} val The value to convert.
 * @returns {boolean} The converted value as a boolean.
 */
function toBool(val) {
    if (typeof val === 'string') {
        return ['true', 'yes', 'y', 'on', '1'].includes(val.toLowerCase());
    }
    if (typeof val === 'number') {
        return val === 1;
    }
    return Boolean(val);
}
exports.toBool = toBool;
