/**
 * Minimal formatter for log messages.
 * @param {object} cfg  Configuration object for the formatter.
 * @returns {FormatPlugin} The minimal formatter function.
 */
export default function minimalFormatter(cfg = {}) {
    const fmCfg = {
        includeTimestamp: false,
        formatTimestamp: (timestamp) => timestamp.toISOString(),
        separator: ' | ',
        ...cfg
    };
    const minimalFmt = (ctx) => {
        const parts = [];
        if (fmCfg.includeTimestamp) {
            parts.push(fmCfg.formatTimestamp(new Date()));
        }
        parts.push(ctx.level.toUpperCase(), ctx.msg);
        return parts.join(fmCfg.separator);
    };
    minimalFmt._cfg = fmCfg;
    return minimalFmt;
}
